@@ -6,7 +6,8 @@ from trytond.i18n import gettext
 from trytond.model import ModelView, Workflow, fields
 from trytond.model.exceptions import AccessError
 from trytond.pool import Pool, PoolMeta
-from trytond.transaction import Transaction
+from trytond.pyson import Eval
+from trytond.transaction import Transaction, without_check_access
 
 
 def process_purchase(func):
@@ -14,12 +15,16 @@ def process_purchase(func):
     def wrapper(cls, invoices):
         pool = Pool()
         Purchase = pool.get('purchase.purchase')
-        with Transaction().set_context(_check_access=False):
+        transaction = Transaction()
+        context = transaction.context
+        with without_check_access():
             purchases = set(
                 p for i in cls.browse(invoices) for p in i.purchases)
         func(cls, invoices)
         if purchases:
-            Purchase.__queue__.process(purchases)
+            with transaction.set_context(
+                    queue_batch=context.get('queue_batch', True)):
+                Purchase.__queue__.process(purchases)
     return wrapper
 
 
@@ -57,23 +62,14 @@ class Invoice(metaclass=PoolMeta):
 
     @classmethod
     def search_purchases(cls, name, clause):
-        return [('lines.origin.purchase' + clause[0].lstrip(name),)
-            + tuple(clause[1:3]) + ('purchase.line',) + tuple(clause[3:])]
+        return [('lines.origin.purchase' + clause[0][len(name):],
+                *clause[1:3], 'purchase.line', *clause[3:])]
 
     @classmethod
     @process_purchase
     def delete(cls, invoices):
         super(Invoice, cls).delete(invoices)
 
-    @classmethod
-    def copy(cls, invoices, default=None):
-        if default is None:
-            default = {}
-        else:
-            default = default.copy()
-        default.setdefault('purchases', None)
-        return super(Invoice, cls).copy(invoices, default=default)
-
     @classmethod
     @ModelView.button
     @Workflow.transition('draft')
@@ -105,6 +101,15 @@ class Invoice(metaclass=PoolMeta):
 class InvoiceLine(metaclass=PoolMeta):
     __name__ = 'account.invoice.line'
 
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        if not cls.origin.domain:
+            cls.origin.domain = {}
+        cls.origin.domain['purchase.line'] = [
+            ('type', '=', Eval('type')),
+            ]
+
     @fields.depends('origin')
     def on_change_with_product_uom_category(self, name=None):
         pool = Pool()
@@ -115,9 +120,19 @@ class InvoiceLine(metaclass=PoolMeta):
         # Use getattr as reference field can have negative id
         if (isinstance(self.origin, PurchaseLine)
                 and getattr(self.origin, 'unit', None)):
-            category = self.origin.unit.category.id
+            category = self.origin.unit.category
         return category
 
+    def get_warehouse(self, name):
+        pool = Pool()
+        PurchaseLine = pool.get('purchase.line')
+        warehouse = super().get_warehouse(name)
+        if (not warehouse
+                and isinstance(self.origin, PurchaseLine)
+                and self.origin.purchase.warehouse):
+            warehouse = self.origin.purchase.warehouse.id
+        return warehouse
+
     @property
     def origin_name(self):
         pool = Pool()
@@ -137,10 +152,14 @@ class InvoiceLine(metaclass=PoolMeta):
     def delete(cls, lines):
         pool = Pool()
         Purchase = pool.get('purchase.purchase')
-        with Transaction().set_context(_check_access=False):
+        transaction = Transaction()
+        context = transaction.context
+        with without_check_access():
             invoices = (l.invoice for l in cls.browse(lines)
                 if l.type == 'line' and l.invoice)
             purchases = set(p for i in invoices for p in i.purchases)
         super(InvoiceLine, cls).delete(lines)
         if purchases:
-            Purchase.__queue__.process(purchases)
+            with transaction.set_context(
+                    queue_batch=context.get('queue_batch', True)):
+                Purchase.__queue__.process(purchases)
