@@ -730,7 +730,11 @@ class Purchase(
 
     def check_for_quotation(self):
         for line in self.lines:
-            if (not line.to_location
+            if (line.quantity or 0) >= 0:
+                location = line.to_location
+            else:
+                location = line.from_location
+            if ((not location or not line.warehouse)
                     and line.product
                     and line.product.type in line.get_move_product_types()):
                 raise PurchaseQuotationError(
@@ -1875,7 +1879,7 @@ class Line(sequence_ordered(), ModelSQL, ModelView):
         quantity = (
             self._get_move_quantity(move_type)
             - self._get_shipped_quantity(move_type))
-        if quantity < 0:
+        if self.unit.round(quantity) < 0:
             warning_name = Warning.format(
                 'check_move_quantity', [self])
             if Warning.check(warning_name):
